import tkinter
import tkinter.font
from pathlib import Path
from datetime import datetime
import sys

class Gui(tkinter.Tk):
    def __init__(self, notes_path: Path, extension: str) -> None:
        super().__init__()

        self.notes_dir_path: Path = notes_path
        self.extension = extension
        self.bind_all('<Control-Return>', self.save_note_and_exit)

        self.title('New Note')

        # Set the logo of the app and make it MacOS compatible
        logo_path: Path = Path(
            Path(__file__).parent, # Get the path to the root of the package
            'assets/logo.png'
            ).absolute()
        logo: tkinter.Image = tkinter.Image('photo', file=f'{logo_path}')
        self.tk.call('wm','iconphoto', self._w, logo)

        # Set the widgets
        self.set_widgets()
        self.input.focus_set()

        # Start the loops
        self.scrollbar_loop()
        self.title_loop()

        self.mainloop()
    
    def set_widgets(self) -> None:

        font_size: tkinter.font.Font = tkinter.font.Font(size=20)

        # Input widget
        self.input = tkinter.Text(
            self,
            highlightthickness=0,
            padx=20,
            pady=20,
            font= font_size,
            height=9,
            width=30,
            )
        self.input.pack(side=tkinter.LEFT)

        # Scrollbar widget
        self.scrollbar = tkinter.Scrollbar()

        # Set the relation between the textbox and the scrollbar
        self.input.config(yscrollcommand=self.scrollbar.set)
        self.scrollbar.config(command=self.input.yview)

        # Clean the menu bar
        self.menu = tkinter.Menu()
        self.config(menu=self.menu)

    def title_loop(self):
        """
        Updates the title text according the first line in the textbox.
        """
        new_title: str = self.input.get('1.0', '1.end')
        if len(new_title) >= 20:
            new_title = new_title[:17] + '...'

        if new_title != '':
            self.title(new_title)
        else:
            self.title('New Note')

        self.after(1, self.title_loop)

    def scrollbar_loop(self):
        """
        Check if the number of lines has exceeded the maximum of the textbox and show or hide a scrollbar as appropriate.
        """
        number_of_lines: int = self.input.get('1.0', 'end-1c').count('\n')

        if number_of_lines > 8:
            self.scrollbar.pack(side=tkinter.RIGHT, fill='y')
        else:
            self.scrollbar.pack_forget()

        self.after(1, self.scrollbar_loop)

    def check_note_path(self, note_path: Path) -> Path:
        if not note_path.is_file():
            return note_path
        
        datetime_now: str = datetime.now().strftime('%H%M%S-%d%m%Y')
        new_filename: str = f'{note_path.stem}-{datetime_now}{note_path.suffix}'

        return self.check_note_path(Path(note_path.parent, new_filename))

    def save_note_and_exit(self, event: tkinter.Event) -> None:
        # Set the note file
        self.notes_dir_path.mkdir(parents=True, exist_ok=True)

        # Check if the first line in the textbox is valid as a filename
        note_filename: str
        if self.input.get('1.0', '1.end') != '':
            note_filename = self.input.get('1.0', '1.end')
        else:
            note_filename = datetime.now().strftime('%H%M%S-%d%m%Y')

        # Get the note path and check if its valid
        note_path: Path = Path(self.notes_dir_path, f'{note_filename}.{self.extension}')
        note_path = self.check_note_path(note_path)

        # Get the textbox data and save it
        note:str = self.input.get('1.0', 'end-1c')

        note_path.touch()
        with open(note_path, 'w') as file:
            file.write(note)
        
        sys.exit(0)



from . import config
from pathlib import Path
from argparse import ArgumentParser, Namespace
from .gui import Gui
import typing
import sys

def main() -> None:
    parser: ArgumentParser = ArgumentParser(
        prog='yaqn',
        description='A markdown quicknote app made in python and gtk.'
    )

    parser.add_argument(
        '--config',
        default=None,
        required=False,
        dest='custom_config_path',
        help='Set a custom path (absolute) to the config file.',
        type=str,
    )

    parser.add_argument(
        '--check',
        default=False,
        required=False,
        dest='check_mode',
        help='Only run the config file checks and restore it if is necessary.',
        action='store_true',
    )

    args: Namespace = parser.parse_args()

    # Check if the custom config path selected by the user is a dir
    custom_config_path: Path | None
    if args.custom_config_path is not None and Path(args.custom_config_path).is_dir():
        custom_config_path = Path(args.custom_config_path)
    else:
        custom_config_path = None

    config_data: dict[str, typing.Any] = config.read_config(custom_config_path)

    # If check mode is activated not start the gui
    if args.check_mode:
        print('[ INFO ] Check mode -> Config checked and operative')
        sys.exit(0)

    gui: Gui = Gui(config_data['notes_path'], config_data['extension'])



from pathlib import Path
import platform
import tomllib
import typing

# Define the default paths
DEFAULT_NOTES_PATH: Path = Path(Path.home(), 'Documents', 'notes')
DEFAULT_EXTENSION: str = 'md'

def init_config(custom_path: Path | None = None) -> Path:
    """
    Check if the config path and file exist and configure a custom path if it is given.
    """
    config_path: Path = get_default_config_path()

    if custom_path is not None:
        config_path = custom_path

    if not config_path.is_dir():
        config_path.mkdir()

    yaqn_path: Path = Path(config_path, 'yaqn')
    yaqn_file: Path = Path(yaqn_path, 'config.toml')

    if not yaqn_path.is_dir():
        yaqn_path.mkdir()

    if not yaqn_file.is_file():
        yaqn_file.touch()

    return yaqn_file

def get_default_config_path() -> Path:
    if platform.system() == 'Windows':
        return Path(Path.home(), 'AppData', 'Roaming')
    else:
        return Path(Path.home(), '.config')

def check_config(config_path: Path) -> None:
    """
    Check if the config file is valid
    """
    with open(config_path, 'rb') as config:
        # Check if the config is a valid toml file
        try:
            loaded_config: dict = tomllib.load(config)
        except tomllib.TOMLDecodeError:
            regenerate_config(config_path)

        # Check if the config is following the expected structure
        match loaded_config:
            case {'notes_path': str(), 'extension': str()}:
                pass
            case _:
                regenerate_config(config_path)


def regenerate_config(config_path: Path) -> None:
    """
    Rewrite the config file with all the defaults.
    """
    with open(config_path, 'w') as config:
        config.writelines([
            f'notes_path = \'{DEFAULT_NOTES_PATH}\'',
            f'\nextension = \'{DEFAULT_EXTENSION}\''
        ])

def read_config(custom_path: Path | None = None) -> dict[str, typing.Any]:
    """
    Read the config file, check it and repair it if is necessary.
    """
    config_path: Path = init_config(custom_path)
    check_config(config_path)

    with open(config_path, 'rb') as config:
        data: dict[str, typing.Any] = tomllib.load(config)
        return {
            'notes_path': Path(data['notes_path']),
            'extension': data['extension']
            }



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['yaqn']

package_data = \
{'': ['*'], 'yaqn': ['assets/*']}

entry_points = \
{'console_scripts': ['yaqn = yaqn.main:main']}

setup_kwargs = {
    'name': 'yaqn',
    'version': '1.0.1',
    'description': 'A quicknotes app made in python and tkinter.',
    'long_description': '# YAQN - Yeet Another Quick Note\n> A fast, easy and OOTB app for taking your notes made in Python.\n\n<img src="./assets/screenshots/app.png" alt="Screenshot of the app" width=542>\n\n## Installation\n```\n> pip install yaqn  # Install from pypi\n> yaqn --check # Run the app and generate the config file\n```\n\n## How to use\nRun the `yaqn` command, it will open the app. To save the note and close the app itself just press `Ctrl + Enter`.\n\n### Open with a keyboard shortcut\nThis can\'t be done automatically by `pip`, please use the correct for your OS:\n- MacOS: Use [Karabiner Elements](https://karabiner-elements.pqrs.org/) and import [this rule](https://github.com/kutu-dev/yaqn/tree/master/assets/karabiner-rules/open-yaqn.json).\n- Windows: _Too complex too configure unfortunately..._\n- Linux, FreeBSD and others depend of the window manager you\'re using.\n\n## Configuration\nYour configuration file is saved by default in:\n- Unix-like: `~/.config/yaqn/config.toml`\n- Windows: `%AppData%\\yaqn\\config.toml`\n\n### Custom config path\nYou can use your custom path for the config using `--config`:\n```\n> yaqn ---config /path/to/config/directory\n```\n### Check the config file integrity\nThe argument `--check` allows you to create, check and repair the config file, just run the command and it will analyze the config:\n```\n> yaqn --check\n[ INFO ] Check mode -> Config checked and operative\n```\n\n### Configuration structure\nAn ordinary config file looks like this:\n```\nnotes_path = \'/path/to/notes/directory\'\nextension = \'md\'\n```\nAnd this is its structure:\n\n| Parameter | Description |\n| --- | --- |\n| `notes_path` | Define the path to the directory the notes will be saved. |\n| `extension` | Define the extension the notes will be saves. _Remember to **not** put `.` before the extension itself._ |\n\n## Author\n\nCreated with :heart: by [Kutu](https://kutu-dev.github.io/).\n> - GitHub - [kutu-dev](https://github.com/kutu-dev)\n> - Twitter - [@kutu-dev](https://twitter.com/kutu_dev)\n\nLogo of the app created by [vladlucha](https://macosicons.com/#/u/vladlucha) (account deleted unfortunately) in [MacOS Icons](https://macosicons.com/#/).',
    'author': 'kutu-dev',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'entry_points': entry_points,
    'python_requires': '>=3.11,<4.0',
}


setup(**setup_kwargs)
